@@ -21,36 +21,57 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <limits.h>
 
-extern int  readsfv(char*, char*, int);
-extern int  newsfv(char**);
-extern void pusage();
+#include "cksfv.h"
 
-int quiet = 0; /* 1 if you want the program to be quiet by default */
+#ifndef PATH_MAX
+#define PATH_MAX (512)
+#endif
+
+/* config variables for other modules */
+int use_basename = 0;
+int be_quiet = 0;
+int be_caseinsensitive = 0;
+int recurse = 0;
 
 int main(int argc, char *argv[])
 {
   int   ch, rval;
-  int   nocase = 0, rsfvflag = 0;
-  char  dir[256] = ".", sfvfile[256];
+  int   rsfvflag = 0;
+  char  dir[PATH_MAX + 1] = ".", sfvfile[PATH_MAX + 1];
+  int follow = 0;
+  int i;
 
-  while ((ch = getopt(argc, argv, "iC:f:qv")) != -1)
+  while ((ch = getopt(argc, argv, "iC:f:qvbrL")) != -1)
     switch (ch) {
     case 'i':
-      nocase = 1;
+      be_caseinsensitive = 1;
       break;
     case 'C':
-      strncpy(dir, optarg, 256);
+      strncpy(dir, optarg, sizeof(dir));
+      dir[sizeof(dir) - 1] = 0;
       break;  
     case 'f':
-      strncpy(sfvfile, optarg, 256);
+      strncpy(sfvfile, optarg, sizeof(sfvfile));
+      sfvfile[sizeof(sfvfile) - 1] = 0;
       rsfvflag = 1;
       break;
+    case 'L':
+      follow = 1;
+      break;
     case 'q':
-      quiet++;
+      be_quiet++;
       break;
     case 'v':
-      quiet = 0;
+      be_quiet = 0;
+      break;
+    case 'b':
+      use_basename = 1;
+      break;
+    case 'r':
+      recurse = 1;
+      rsfvflag = 1;
       break;
     case '?':
     default:
@@ -60,7 +81,10 @@ int main(int argc, char *argv[])
   argv += optind;
 
   if (rsfvflag == 1) {
-    rval = readsfv(sfvfile, dir, nocase);
+    if (recurse == 1)
+      rval = recursivereadsfv(dir, follow, argc, argv);
+    else
+      rval = readsfv(sfvfile, dir, argc, argv);
   } else {
     if (argc < 1) {
       pusage();
@@ -69,5 +93,10 @@ int main(int argc, char *argv[])
     rval = newsfv(argv);
   }
 
+  if (!TOTALLY_QUIET && recurse && sfv_broken) {
+    fprintf(stderr, "\nList of sfv files with broken files (or broken sfv files):\n");
+    for (i = 0; i < sfv_broken; i++)
+      fprintf(stderr, "%s\n", sfv_broken_list[i]);
+  }
   exit(rval);
 }
