@@ -16,41 +16,75 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
+#include <stdlib.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <libgen.h>
 
-extern void pnsfv_head();
-extern void pfileinfo(char**);
-extern void pcrc(char*, unsigned long);
-extern int  crc32(int, unsigned long*, unsigned long*);
+#include "cksfv.h"
+#include "config.h"
+
+#ifndef O_LARGEFILE
+#define O_LARGEFILE (0)
+#endif
 
 int newsfv(char **argv)
 {
-  int           fd, rval = 0;
-  char          *fn;
-  unsigned long len, val;
-  
+  int fd, rval = 0;
+  char *fn;
+  uint32_t val;
+  char *tmpname;
+  struct stat st;
+
   pnsfv_head();
   pfileinfo(argv);
-  
+
   while (*argv) {
     fn = *argv++;
-    if ((fd = open(fn, O_RDONLY, 0)) < 0) {
-      fprintf(stderr, "cksfv: %s: %s\n", fn, strerror(errno)); 
+    if ((fd = open(fn, O_RDONLY | O_LARGEFILE, 0)) < 0) {
+      if (!TOTALLY_QUIET)
+	fprintf(stderr, "cksfv: %s: %s\n", fn, strerror(errno)); 
       rval = 1;
       continue;
     }
-    
-    if (crc32(fd, &val, &len)) {
-      fprintf(stderr, "cksfv: %s: %s\n", fn, strerror(errno)); 
+    if (fstat(fd, &st)) {
+      if (!TOTALLY_QUIET)
+	fprintf(stderr, "cksfv: can not fstat %s: %s\n", fn, strerror(errno)); 
       rval = 1;
-    } else
-      pcrc(fn, val);
+      goto next;
+    }
+    if (S_ISDIR(st.st_mode)) {
+      if (!TOTALLY_QUIET)
+	fprintf(stderr, "cksfv: %s: Is a directory\n", fn);
+      rval = 1;
+      goto next;
+    }
+    if (crc32(fd, &val)) {
+      if (!TOTALLY_QUIET)
+	fprintf(stderr, "cksfv: %s: %s\n", fn, strerror(errno)); 
+      rval = 1;
+    } else {
+      if (use_basename) {
+	if ((tmpname = strdup(fn)) == NULL) {
+	  if (!TOTALLY_QUIET)
+	    fprintf(stderr, "out of memory\n");
+	  exit(-1);
+	}
+	pcrc(basename(tmpname), val);
+	free(tmpname);
+      } else {
+	pcrc(fn, val);
+      }
+    }
+  next:
     close(fd);
   }
 
-  return(rval);
+  return rval;
 }
